from uuid import uuid4
try:
    from Cryptodome.PublicKey import RSA
    from Cryptodome.Random import get_random_bytes
except:
    from Crypto.PublicKey import RSA
    from Crypto.Random import get_random_bytes
from sqlalchemy.engine import create_engine
from sqlalchemy.sql.schema import MetaData
from website.models import db, AccessCode, User
from app import app
from config import SQLALCHEMY_DATABASE_URI
import os

print("Important: Only run once and copy the following output to store it in a safe place\n")

if not os.path.isfile("output/private_key.pem") and not os.path.isfile("output/public_key.pem"):
    # generate keypair
    keyPair = RSA.generate(4096)

    # create output path if not existent
    if not os.path.exists('output'):
        os.mkdir('output')

    # save public key
    pubKey = keyPair.publickey()
    #print(f"Public key:  (n={hex(pubKey.n)}, e={hex(pubKey.e)})")
    pubKeyPEM = pubKey.exportKey()
    with open("output/public_key.pem", "wb") as f:
        f.write(pubKeyPEM)

    # save private key
    #print(f"Private key: (n={hex(pubKey.n)}, d={hex(keyPair.d)})")
    privKeyPEM = keyPair.exportKey()
    with open("output/private_key.pem", "wb") as f:
        f.write(privKeyPEM)

# save signing key
if not os.path.isfile("output/signing_key.aes"):
    key = get_random_bytes(32)
    with open("output/signing_key.aes", "wb") as f:
        f.write(key)

# set up db and create access code for first user
engine = create_engine(SQLALCHEMY_DATABASE_URI)

with app.app_context():
    MetaData().create_all(engine, tables=[
        AccessCode.__table__, User.__table__])
    code = uuid4().hex
    db.session.add(AccessCode(code=code, scope="admin"))
    db.session.commit()
    new_code = AccessCode.query.filter_by(code=code).first()
    print("\nAdmin Access Code:\t", new_code.code)
