library digital_3g_common;

import 'dart:convert';

import 'package:http/http.dart' as http;
import 'package:logging/logging.dart';

class VerificationApi {
  final Logger _log = Logger('VerificationApi');

  final http.Client _client;
  final String baseUrl;

  VerificationApi(this._client, this.baseUrl);

  Future<VerificationResponse> check(String data) async {
    final response = await _client.post(
      Uri.parse('$baseUrl/check'),
      body: json.encode(data),
      headers: {
        'Content-Type': 'application/json',
      },
    );
    _log.finer('check received ${response.statusCode} status');
    if (response.statusCode == 200) {
      _log.finest(response.body);
      return VerificationResponse.fromMapNew(json.decode(response.body));
    } else if (response.statusCode == 404) {
      _log.fine('no color available');
      _log.finest(response.body);
      return VerificationResponse.fromMapNew(json.decode(response.body));
    } else {
      _log.severe(response.body);
      return VerificationResponse.empty();
    }
  }
}

class VerificationResponse {
  final bool verified;
  final int? color;
  final bool? inverse;

  final DateTime timestamp;

  VerificationResponse({required this.verified, this.color, this.inverse, required this.timestamp});

  factory VerificationResponse.empty() =>
      VerificationResponse(verified: false, timestamp: DateTime.now());

  factory VerificationResponse.fromMap(Map map) => VerificationResponse(
        verified: map['verified'],
        color: map['color'] != null ? int.tryParse(map['color'], radix: 16) : null,
        inverse: map['inverse'],
        timestamp: DateTime.tryParse(map['timestamp']) ?? DateTime(1971),
      );

  factory VerificationResponse.fromMapNew(Map map) => VerificationResponse(
        verified: map['verified'],
        color: map['color'] != null ? int.tryParse(map['color'], radix: 16) : null,
        inverse: map['inverse'],
        timestamp: DateTime.now(),
      );

  Map<String, dynamic> get toMap => {
        'verified': verified,
        'color': color?.toRadixString(16),
        'inverse': inverse,
        'timestamp': timestamp.toIso8601String(),
      };

  @override
  String toString() => '[VerificationResponse $toMap]';
}
