import 'dart:convert';

import 'package:flutter/widgets.dart';
import 'package:http/http.dart' as http;

import '../models/signed_ticket.dart';

class UserApi {
  final String _baseUrl;
  final http.Client _client;

  UserApi(this._client, this._baseUrl);

  http.Client get client => _client;

  Future<Color?> getCurrentColor() async {
    final response = await _client.get(Uri.parse('$_baseUrl/current_color'));

    if (response.statusCode == 200) {
      return Color(int.parse(json.decode(response.body)['color'] as String, radix: 16));
    }
  }

  Future<SignedTicket?> signTicket(String ticket) async {
    final response = await _client.post(
      Uri.parse('$_baseUrl/sign'),
      body: json.encode(ticket),
      headers: {'Content-Type': 'application/json'},
    );

    if (response.statusCode == 200) {
      return SignedTicket.fromString(json.decode(response.body)['combined']);
    } else {
      return null;
    }
  }
}
