import 'package:flutter/material.dart';
import 'package:flutter_colorpicker/flutter_colorpicker.dart';

import '../models/ribbon_color.dart';
import 'content_dialog.dart';

class ColorDialog extends StatefulWidget {
  final RibbonColor? color;

  const ColorDialog({Key? key, this.color}) : super(key: key);

  @override
  _ColorDialogState createState() => _ColorDialogState();
}

class ColorDialogResult {
  final bool save;
  final RibbonColor color;

  ColorDialogResult({required this.save, required this.color});
}

class _ColorDialogState extends State<ColorDialog> {
  RibbonColor? _color;

  @override
  Widget build(BuildContext context) {
    return ContentDialog(
      title: Text(widget.color != null ? 'Farbe bearbeiten' : 'Farbe hinzufügen'),
      children: [
        Padding(
          padding: const EdgeInsets.only(bottom: 8.0),
          child: TextField(
            decoration: const InputDecoration(labelText: 'Name (optional)'),
            onChanged: (String input) => _color!.name = input,
            controller: TextEditingController(text: _color!.name),
          ),
        ),
        ColorPicker(
          pickerColor: Color(_color!.hex),
          onColorChanged: (Color c) => setState(() => _color!.hex = c.value),
          showLabel: false,
          pickerAreaHeightPercent: 0.8,
          enableAlpha: false,
        ),
        TextFormField(
          decoration: const InputDecoration(labelText: 'Hex-Wert'),
          onFieldSubmitted: (String input) =>
              setState(() => _color!.hex = int.parse(input, radix: 16) + 0xFF000000),
          controller: TextEditingController(
              text: (_color!.hex - 0xFF000000).toRadixString(16).padLeft(6, '0')),
        ),
        CheckboxListTile(
          title: const Text('Invertiertes Logo'),
          value: _color!.inverse,
          onChanged: (bool? input) {
            if (input != null) {
              setState(() => _color!.inverse = input);
            }
          },
        ),
        const Divider(),
      ],
      actions: [
        ElevatedButton(
          onPressed: () =>
              Navigator.of(context).pop(ColorDialogResult(save: false, color: _color!)),
          child: const Text('Abbrechen'),
        ),
        ElevatedButton(
          onPressed: () => Navigator.of(context).pop(ColorDialogResult(save: true, color: _color!)),
          child: const Text('Speichern'),
        ),
      ],
    );
  }

  @override
  void initState() {
    _color = widget.color ?? RibbonColor(hex: 0xFF000000);
    super.initState();
  }
}
