import 'package:clipboard/clipboard.dart';
import 'package:csv/csv.dart';
import 'package:flutter/material.dart';

import '../models/access_code.dart';
import 'content_dialog.dart';

class CodeCsvConfig {
  bool showCode;
  bool showScope;

  CodeCsvConfig({
    this.showCode = true,
    this.showScope = false,
  });
}

class CsvDialog extends StatefulWidget {
  final List<AccessCode> filteredCodes;
  final List<int> markedCodes;

  const CsvDialog({
    Key? key,
    this.filteredCodes = const [],
    this.markedCodes = const [],
  }) : super(key: key);

  @override
  _CsvDialogState createState() => _CsvDialogState();
}

class _CsvDialogState extends State<CsvDialog> {
  final converter = const ListToCsvConverter();

  final csvConfig = CodeCsvConfig();

  String? csv;

  TextEditingController controller = TextEditingController();

  @override
  Widget build(BuildContext context) {
    return ContentDialog(title: const Text('CSV-Liste'), children: [
      CheckboxListTile(
        value: csvConfig.showCode,
        onChanged: (bool? input) {
          setState(() => csvConfig.showCode = input ?? false);
          onUpdateCsv(csvConfig);
        },
        title: const Text('Code zeigen'),
      ),
      CheckboxListTile(
        value: csvConfig.showScope,
        onChanged: (bool? input) {
          setState(() => csvConfig.showScope = input ?? false);
          onUpdateCsv(csvConfig);
        },
        title: const Text('Rolle zeigen'),
      ),
      Padding(
        padding: const EdgeInsets.all(8.0),
        child: TextField(
          maxLines: 12,
          controller: controller,
        ),
      ),
    ], actions: [
      ElevatedButton(
        onPressed: () {
          try {
            FlutterClipboard.copy(csv!);
          } catch (e) {
            ScaffoldMessenger.of(context).showSnackBar(const SnackBar(
              content: Text(
                'Da ist etwas schiefgelaufen, probiere es noch mal oder wende dich an den Administrator',
              ),
            ));
          }
          ScaffoldMessenger.of(context).showSnackBar(
            const SnackBar(content: Text('CSV in die Zwischenablage kopiert')),
          );
          Navigator.of(context).pop();
        },
        child: const Text('Kopieren'),
      ),
    ]);
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    onUpdateCsv(csvConfig);
  }

  void onUpdateCsv(CodeCsvConfig config) {
    setState(() => csv = converter.convert(widget.filteredCodes
            .where((AccessCode code) =>
                widget.markedCodes.isNotEmpty ? widget.markedCodes.contains(code.id) : true)
            .map<List<dynamic>>(
          (AccessCode e) {
            var values = [];
            if (config.showCode) {
              values.add(e.code);
            }
            if (config.showScope) {
              values.add(e.scope);
            }
            return values;
          },
        ).toList()));
    controller.text = csv!;
  }
}
